/**
 * $RCSfile$
 * $Revision: 1722 $
 * $Date: 2005-07-28 15:19:16 -0700 (Thu, 28 Jul 2005) $
 *
 * Copyright (C) 2004 Jive Software. All rights reserved.
 *
 * This software is published under the terms of the GNU Public License (GPL),
 * a copy of which is included in this distribution.
 */

package org.jivesoftware.messenger.plugin;

import org.jivesoftware.messenger.container.Plugin;
import org.jivesoftware.messenger.container.PluginManager;
import org.jivesoftware.messenger.user.UserManager;
import org.jivesoftware.messenger.user.UserNotFoundException;
import org.jivesoftware.messenger.user.UserAlreadyExistsException;
import org.jivesoftware.messenger.user.User;
import org.jivesoftware.messenger.XMPPServer;
import org.jivesoftware.util.JiveGlobals;
import org.xmpp.packet.JID;

import java.io.File;

/**
 * Plugin that includes a servlet that provides information about the presence type of the
 * users in the server. For security reasons, the XMPP spec does not allow anyone to see
 * the presence of any user. Only the users that are subscribed to the presence of other
 * users may see their presences.<p>
 *
 * However, in order to make the servlet more useful it is possible to configure this plugin
 * so that anyone or only the users that are subscribed to a user presence may see the presence
 * of other users.<p>
 *
 * Currently, the servlet provides information about user presences in two formats. In XML format
 * or using images.
 *
 * @author Gaston Dombiak
 */
public class UserAdminPlugin implements Plugin {

    private UserManager userManager;
    private String hostname;

    public void initializePlugin(PluginManager manager, File pluginDirectory) {
        XMPPServer server = XMPPServer.getInstance();
        userManager = server.getUserManager();
        hostname = server.getServerInfo().getName();
    }

    public void destroyPlugin() {
        userManager = null;
    }


    public void createUser(String username, String password, String name, String email) throws UserAlreadyExistsException{
        userManager.createUser(username, password, name, email);                
    }
    
    public void deleteUser(String jid) throws UserNotFoundException{
        User user = fetchUser(jid);
        userManager.deleteUser(user);
    }
    
    public void updateUser(String jid, String password, String name, String email) throws UserNotFoundException{
        User user = fetchUser(jid);
        user.setPassword(password);
        user.setName(name);
        user.setEmail(email);
    }
    
    /**
     * Returns the the requested user or <tt>null</tt> 
     * if there are any problems that don't throw an error.
     * 
     *
     * @param jid the bare JID of the entity whose presence is being probed.
     * @return the requested user.
     * @throws UserNotFoundException  if the requested user
     *         does not exist in the local server.
     */

    private User fetchUser(String jid) throws UserNotFoundException {
        JID targetJID = new JID(jid);
        // Check that the sender is not requesting information of a remote server entity
        if (targetJID.getDomain() == null || XMPPServer.getInstance().isRemote(targetJID)) {
            throw new UserNotFoundException("Domain does not matches local server domain");
        }
        if (!hostname.equals(targetJID.getDomain())) {
            // Sender is requesting information about component presence
            // TODO Implement this
            throw new UserNotFoundException("Presence of components not supported yet!");
        }
        if (targetJID.getNode() == null) {
            // Sender is requesting presence information of an anonymous user
            throw new UserNotFoundException("Username is null");
        }
        User user = userManager.getUser(targetJID.getNode());
        return user;
        
    }
    
}
